"""
HoneyBee Processors Module

Provides unified interfaces for different data modality processors.
"""

from .clinical import ClinicalProcessor
from .pathology_processor import PathologyProcessor
from .radiology import RadiologyProcessor


__all__ = [
    "ClinicalProcessor",
    "PathologyProcessor",
    "RadiologyProcessor",
]
